﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace valasztas
{

    class Candidate
    {
        public int AreaCode { get; set; }
        public int Vote { get; set; }
        public string Fname { get; set; }
        public string Sname { get; set; }
        public string Part { get; set; }
        public string Name()
        {
            return Fname + " " + Sname;
        }

        public Candidate(string[] adat)
        {
            AreaCode = int.Parse(adat[0]);
            Vote = int.Parse(adat[1]);
            Fname = adat[2];
            Sname = adat[3];
            Part = adat[4] == "-" ? "Független" : adat[4];
        }
    }

    class Program
    {
        static void Main(string[] args)
        {
            //0. értékadás, pártok listája
            Dictionary<string, string> part = new Dictionary<string, string>();
            part.Add("GYEP", "Gyümölcsevők Pártja");
            part.Add("HEP", "Húsevők Pártja");
            part.Add("TISZ", "Tejivók Szövetsége");
            part.Add("ZEP", "Zöldségevők Pártja");
            part.Add("Független", "Független jelöltek");


            List<Candidate> candidates = new List<Candidate>();

            int fSzám = 1;

            //1. Adatok beolvasása fájlból 
            Console.WriteLine("\n{0}. feladat", fSzám++);

            Feladat01(candidates);

            //2. Megszámlálás, elemszám
            Console.WriteLine("\n{0}. feladat", fSzám++);

            Console.WriteLine("A helyhatósági választáson {0} képviselőjelölt indult.", Feladat02(candidates));
            //egyszerűbben, a lista tulajdonságával:
            //Console.WriteLine("A helyhatósági választáson {0} képviselőjelölt indult.", candidates.Count);

            //3. Keresés
            Console.WriteLine("\n{0}. feladat", fSzám++);

            Console.WriteLine(Feladat03(candidates));

            //4. Összegzés
            Console.WriteLine("\n{0}. feladat", fSzám++);

            int leadott = Feladat04(candidates);
            Console.WriteLine("A választáson {0} állampolgár, a jogosultak {1:P2}-a vett részt.", leadott, 1.0 * leadott / 12345);

            //5. Részösszegek - csoportonkénti aggregálás
            Console.WriteLine("\n{0}. feladat", fSzám++);

            Dictionary<string, int> szavazatok = Feladat05(candidates, part);
            foreach (string r in szavazatok.Keys)
            {
                Console.WriteLine("{0} = {1:P2}", r, szavazatok[r] * 1.0 / leadott);
            }

            //6. Maximumhelyek kigyűjtése
            Console.WriteLine("\n{0}. feladat", fSzám++);

            List<string> legjobbak = Feladat06(candidates);
            foreach (string l in legjobbak)
            {
                Console.WriteLine(l);
            }

            //7. Csoportonkénti maximumok rendezése
            Console.WriteLine("\n{0}. feladat", fSzám++);

            string[] nyertesek = Feladat07(candidates);
            StreamWriter sw = new StreamWriter("kepviselok.txt");
            for (int i = 1; i < 9; i++)
            {
                sw.WriteLine(i + ": " + nyertesek[i]);
            }
            sw.Close();
            Console.WriteLine("kész");

            Console.ReadKey();
        }

        private static void Feladat01(List<Candidate> candidates)
        {
            StreamReader sr = new StreamReader("szavazatok.txt");
            while (!sr.EndOfStream)
            {
                candidates.Add(new Candidate(sr.ReadLine().Split(' ')));
            }
            sr.Close();
        }

        private static int Feladat02(List<Candidate> candidates)
        {
            int db = 0;
            foreach (Candidate c in candidates)
            {
                db++;
            }
            return db;
        }

        private static string Feladat03(List<Candidate> candidates)
        {
            Console.Write("Adja meg - szóközzel elválasztva - a keresett képviselő vezeték- és keresztnevét: ");
            string name = Console.ReadLine();
            int pointer = 0;
            while (pointer < candidates.Count && name != candidates[pointer].Name())
            {
                pointer++;
            }

            if (pointer < candidates.Count)
            {
                return String.Format("A képviselő {0} szavazatot kapott.", candidates[pointer].Vote);
            }
            else
            {
                return String.Format("Ilyen nevű képviselőjelölt nem szerepel a nyilvántartásban!");
            }
        }

        private static int Feladat04(List<Candidate> candidates)
        {
            int sum = 0;
            foreach (Candidate c in candidates)
            {
                sum += c.Vote;
            }
            return sum;
        }

        private static Dictionary<string, int> Feladat05(List<Candidate> candidates, Dictionary<string, string> part)
        {
            Dictionary<string, int> voteSums = new Dictionary<string, int>();
            foreach (string p in part.Keys)
            {
                voteSums.Add(part[p], 0);
            }

            int szum = 0;
            foreach (Candidate c in candidates)
            {
                voteSums[part[c.Part]] += c.Vote;
                szum += c.Vote;
            }
            return voteSums;
        }

        private static List<string> Feladat06(List<Candidate> candidates)
        {
            List<string> maxs = new List<string>();
            int max = -1;
            foreach (Candidate c in candidates)
            {
                if (c.Vote > max)
                {
                    max = c.Vote;
                }
            }
            Console.WriteLine("Legtöbb szavazatot kapta:");
            foreach (Candidate c in candidates)
            {
                if (c.Vote == max)
                {
                    maxs.Add(String.Format("\t{0} - {1}", c.Name(), c.Part));
                }
            }
            return maxs;
        }

        private static string[] Feladat07(List<Candidate> candidates)
        {
            int[] localMax = new int[9];
            string[] winner = new string[9];
            for (int i = 0; i < 9; i++)
            {
                localMax[i] = 0;
            }
            foreach (Candidate c in candidates)
            {
                if (c.Vote > localMax[c.AreaCode])
                {
                    localMax[c.AreaCode] = c.Vote;
                    winner[c.AreaCode] = String.Format("{0} {1}", c.Name(), c.Part);
                }
            }

            return winner;
        }

    }
}
