﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace valasztas
{

    class Candidate
    {
        public int AreaCode { get; set; }
        public int Vote { get; set; }
        public string Fname { get; set; }
        public string Sname { get; set; }
        public string Part { get; set; }
        public string Name()
        {
            return Fname + " " + Sname;
        }

        public Candidate(string[] adat)
        {
            AreaCode = int.Parse(adat[0]);
            Vote = int.Parse(adat[1]);
            Fname = adat[2];
            Sname = adat[3];
            Part = adat[4] == "-" ? "Független" : adat[4];
        }
    }

    class Program
    {

        static void Main(string[] args)
        {
            //0. értékadás, pártok listája
            Dictionary<string, string> part = new Dictionary<string, string>();
            part.Add("GYEP", "Gyümölcsevők Pártja");
            part.Add("HEP", "Húsevők Pártja");
            part.Add("TISZ", "Tejivók Szövetsége");
            part.Add("ZEP", "Zöldségevők Pártja");
            part.Add("Független", "Független jelöltek");

            List<Candidate> candidates = new List<Candidate>();

            int fSzám = 1;

            //1. Adatok beolvasása fájlból 
            Console.WriteLine("\n{0}. feladat", fSzám++);

            List<string> sorok = File.ReadAllLines("szavazatok.txt").ToList();  //sorok beolvasása a fájlból
            sorok.ForEach(v => candidates.Add(new Candidate(v.Split(' '))));    //az egyes sorok átadása Candidate konstruktorának, az objektum hozzáadása a candidates listához

            //2. Megszámlálás, elemszám
            Console.WriteLine("\n{0}. feladat", fSzám++);

            Console.WriteLine("A helyhatósági választáson {0} képviselőjelölt indult.", candidates.Count());

            //3. Keresés
            Console.WriteLine("\n{0}. feladat", fSzám++);

            Console.Write("Adja meg - szóközzel elválasztva - a keresett képviselő vezeték- és keresztnevét: ");
            string name = Console.ReadLine();

            if (candidates.Find(c => c.Name() == name) == null)
            {
                Console.WriteLine("Ilyen nevű képviselőjelölt nem szerepel a nyilvántartásban!");
            }
            else
            {
                int vote = candidates.Find(c => c.Name() == name).Vote;
                Console.WriteLine("A képviselő {0} szavazatot kapott.", vote);
            }

            //4. Összegzés
            Console.WriteLine("\n{0}. feladat", fSzám++);

            int szum = candidates.Sum(c => c.Vote);

            Console.WriteLine("A választáson {0} állampolgár, a jogosultak {1:P2}-a vett részt.", szum, 1.0 * szum / 12345);

            //5. Részösszegek - csoportonkénti aggregálás
            Console.WriteLine("\n{0}. feladat", fSzám++);

            var partszum = candidates.GroupBy(c => c.Part).Select(p => new { PartName = part[p.Key], PartVote = p.Sum(c => c.Vote) }).ToList();
            //pártonkénti csoportosítás, csoportonként a párt neve és szavazatok összege

            partszum.ForEach(p => Console.WriteLine("{0}: {1:P2}", p.PartName, p.PartVote * 1.0 / candidates.Sum(c => c.Vote)));
            //az eredmény kiírása

            //6. Maximumhelyek kigyűjtése
            Console.WriteLine("\n{0}. feladat", fSzám++);

            Console.WriteLine("Legtöbb szavazatot kapta:");

            int max = candidates.Max(m => m.Vote);      //legnagyobb értél
            List<Candidate> maxok = candidates.Where(c => c.Vote == max).ToList();  //a max. szavazatot kapottak listája
            maxok.ForEach(e => Console.WriteLine("\t" + e.Name() + " " + e.Part));  //az eredmény kiírása

            //kiírás nélkül egy másik megoldás:
            List<Candidate> rendezett = candidates.OrderByDescending(c => c.Vote).ToList(); //rendezés szavazatok szerint csökkenően
            int maxokszáma = candidates.Count(c => c.Vote == max); //ennyien kaptak max szavazatot
            List<Candidate> top = rendezett.Take(maxokszáma).ToList();  //A lista elejéről vesz megfelelő számú rekorot

            //7. Csoportonkénti maximumok rendezése
            Console.WriteLine("\n{0}. feladat", fSzám++);

            StreamWriter sw = new StreamWriter("kepviselok.txt");

            var nyertesek = candidates.GroupBy(c => c.AreaCode).Select(legjobb => legjobb.OrderByDescending(v => v.Vote).First()).ToList();
                //Kerületenkénti csoportosítás egyes csoportjaira a szavazatszám szerinti rendezés után az első kiválasztása
            nyertesek.OrderBy(a => a.AreaCode).ToList();    //Az eredmény rendezése kerület szerint
            nyertesek.ForEach(n => sw.WriteLine("{0}. {1} {2}", n.AreaCode, n.Name(), n.Part)); //Az eredmény fájlba írása


            sw.Close();
            Console.WriteLine("kész");
            Console.ReadKey();
        }


    }
}
