﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace valasztas
{

    class Candidate
    {
        public int AreaCode { get; set; }
        public int Vote { get; set; }
        public string Fname { get; set; }
        public string Sname { get; set; }
        public string Part { get; set; }
        public string Name()
        {
            return Fname + " " + Sname;
        }

        public Candidate(string[] adat)
        {
            AreaCode = int.Parse(adat[0]);
            Vote = int.Parse(adat[1]);
            Fname = adat[2];
            Sname = adat[3];
            Part = adat[4] == "-" ? "Független" : adat[4];
        }
    }

    class Program
    {

        static void Main(string[] args)
        {
            //0. értékadás, pártok listája
            Dictionary<string, string> part = new Dictionary<string, string>();
            part.Add("GYEP", "Gyümölcsevők Pártja");
            part.Add("HEP", "Húsevők Pártja");
            part.Add("TISZ", "Tejivók Szövetsége");
            part.Add("ZEP", "Zöldségevők Pártja");
            part.Add("Független", "Független jelöltek");

            List<Candidate> candidates = new List<Candidate>();

            int fSzám = 1;

            //1. Adatok beolvasása fájlból 
            Console.WriteLine("\n{0}. feladat", fSzám++);

            File.ReadAllLines("szavazatok.txt").ToList().ForEach(v => candidates.Add(new Candidate(v.Split(' '))));

            //2. Megszámlálás, elemszám
            Console.WriteLine("\n{0}. feladat", fSzám++);

            Console.WriteLine("A helyhatósági választáson {0} képviselőjelölt indult.", candidates.Count());

            //3. Keresés
            Console.WriteLine("\n{0}. feladat", fSzám++);

            Console.Write("Adja meg - szóközzel elválasztva - a keresett képviselő vezeték- és keresztnevét: ");
            string name = Console.ReadLine();
            Console.WriteLine(candidates.Find(c => c.Name() == name) == null ? "Ilyen nevű képviselőjelölt nem szerepel a nyilvántartásban!" : String.Format("A képviselő {0} szavazatot kapott.", candidates.Find(c => c.Name() == name).Vote));

            //4. Összegzés
            Console.WriteLine("\n{0}. feladat", fSzám++);

            Console.WriteLine("A választáson {0} állampolgár, a jogosultak {1:P2}-a vett részt.", candidates.Sum(c => c.Vote), 1.0 * candidates.Sum(c => c.Vote) / 12345);

            //5. Részösszegek - csoportonkénti aggregálás
            Console.WriteLine("\n{0}. feladat", fSzám++);

            candidates.GroupBy(c => c.Part).Select(p => new { PartName = part[p.Key], PartVote = p.Sum(c => c.Vote) }).ToList()
                .ForEach(p => Console.WriteLine("{0}: {1:P2}", p.PartName, p.PartVote * 1.0 / candidates.Sum(c => c.Vote)));

            //6. Maximumhelyek kigyűjtése
            Console.WriteLine("\n{0}. feladat", fSzám++);

            Console.WriteLine("Legtöbb szavazatot kapta:");
            //Maximum értékkel egyező szavazatot kapók adatai
            candidates.Where(c => c.Vote == candidates.Max(m => m.Vote)).ToList().ForEach(e => Console.WriteLine("\t" + e.Name() + " " + e.Part));

            //Szavazatok szerint rendezett adatok közül a maximumok számának megfelelő mennyiségű adat listázása
            var top = candidates.OrderByDescending(c => c.Vote).Take(candidates.Count(c => c.Vote == candidates.Max(m => m.Vote))).ToList();

            //7. Csoportonkénti maximumok rendezése
            Console.WriteLine("\n{0}. feladat", fSzám++);

            StreamWriter sw = new StreamWriter("kepviselok.txt");

            //Csoportonkénti rendezés első értékeinek kigyűjtése
            candidates.GroupBy(c => c.AreaCode)
                .Select(legjobb => legjobb.OrderByDescending(v => v.Vote).First())   //Csoportonkénti legjobb
                .OrderBy(a => a.AreaCode).ToList()    //Az eredmény rendezése kerület szerint
                .ForEach(n => sw.WriteLine("{0}. {1} {2}", n.AreaCode, n.Name(), n.Part)); //Az eredmény fájlba írása

            //A kerületi maximumokat elérők kigyűjtése (másik megoldási elv)
            candidates.GroupBy(c => c.AreaCode).Select(best =>
                new
                {
                    Area = best.Key,        //kerület kódja
                    Win = candidates.Where(c =>
                        (c.Vote == candidates
                            .Where(m => m.AreaCode == best.Key)             //a szavazatszám az egyik kerületben
                            .Max(m => m.Vote) &&                            //a legnagyobb érték
                         c.AreaCode == best.Key))                                                         // és a jelölt kerülete épp ez a kerület
                    .Select(c => String.Format("{0}. {1} {2}", c.AreaCode, c.Name(), c.Part)).ToList().Single() //A kiírás formátumát adja meg eredményként
                }
                ).OrderBy(a => a.Area).ToList();  //az eredmény rendezése (nincs fájlbaírás)


            sw.Close();
            Console.WriteLine("kész");
            Console.ReadKey();
        }


    }
}
