﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace valasztas
{

    class Candidate
    {
        public int AreaCode { get; set; }
        public int Vote { get; set; }
        public string Fname { get; set; }
        public string Sname { get; set; }
        public string Part { get; set; }
        public string Name()
        {
            return Fname + " " + Sname;
        }

        public Candidate(string[] adat)
        {
            AreaCode = int.Parse(adat[0]);
            Vote = int.Parse(adat[1]);
            Fname = adat[2];
            Sname = adat[3];
            Part = adat[4] == "-" ? "Független" : adat[4];
        }
    }

    class Program
    {
        static void Main(string[] args)
        {
            //0. értékadás, pártok listája
            Dictionary<string, string> part = new Dictionary<string, string>();
            part.Add("GYEP", "Gyümölcsevők Pártja");
            part.Add("HEP", "Húsevők Pártja");
            part.Add("TISZ", "Tejivók Szövetsége");
            part.Add("ZEP", "Zöldségevők Pártja");
            part.Add("Független", "Független jelöltek");

            int fSzám = 1;

            //1. Adatok beolvasása fájlból 
            Console.WriteLine("\n{0}. feladat", fSzám++);

            List<Candidate> candidates = (from line in File.ReadAllLines("szavazatok.txt")  //a beolvasott fájl minden sorából
                                          select new Candidate(line.Split(' '))             //hozzon kétre új Candidate objektumot
                                         ).ToList();                                        //és tegye egy listába

            //2. Megszámlálás, elemszám
            Console.WriteLine("\n{0}. feladat", fSzám++);

            Console.WriteLine("A helyhatósági választáson {0} képviselőjelölt indult.", (from c in candidates select c).Count());

            //3. Keresés
            Console.WriteLine("\n{0}. feladat", fSzám++);

            Console.Write("Adja meg - szóközzel elválasztva - a keresett képviselő vezeték- és keresztnevét: ");
            string name = Console.ReadLine();

            var find = from n in candidates
                       where n.Name() == name
                       select n.Vote;
            try
            {
                Console.WriteLine("A képviselő {0} szavazatot kapott.", find.Single()); //Tudom, hogy egyetlen megoldás lehet.
            }
            catch (Exception)
            {
                Console.WriteLine("Ilyen nevű képviselőjelölt nem szerepel a nyilvántartásban!"); //Ha két azonos név lenne, akkor is ezt írná ki.
            }


            //4. Összegzés
            Console.WriteLine("\n{0}. feladat", fSzám++);

            int leadott = (from s in candidates
                           select s.Vote).Sum();

            Console.WriteLine("A választáson {0} állampolgár, a jogosultak {1:P2}-a vett részt.", leadott, 1.0 * leadott / 12345);

            //5. Részösszegek - csoportonkénti aggregálás
            Console.WriteLine("\n{0}. feladat", fSzám++);

            var rate = from c in candidates
                       group c by c.Part into partok
                       select new
                       {
                           Part = partok.Key,
                           Vote = (from x in candidates
                                   where x.Part == partok.Key  //a csoport kulcsának megfelelő listaelemek szűrése
                                   select x.Vote
                                   ).Sum()                         //... és összegzése
                       };

            //részben lambdával egyszerűbb
            //var rate = from c in candidates
            //           group c by c.Part into partok
            //           select new
            //           {
            //               Part = partok.Key,
            //               Vote = partok.Sum(x => x.Vote)
            //           };


            foreach (var r in rate)
            {
                Console.WriteLine("{0}: {1:P2}", part[r.Part], r.Vote * 1.0 / leadott);
            }

            //6. Maximumhelyek kigyűjtése
            Console.WriteLine("\n{0}. feladat", fSzám++);

            int leg = (from v in candidates
                       select v.Vote).Max(); //legnagyobb szavazatszám megadása

            var legek = from c in candidates
                        where c.Vote == (leg)    //a legnagyobb szavazatszánot elérők
                        select new
                        {
                            Name = c.Name(),
                            Part = c.Part
                        };
            foreach (var m in legek)
            {
                Console.WriteLine("\t{0} - {1}", m.Name, m.Part);
            }

            // ugyanez rendezéssel, az elsők kigyűjtésével
            var ordered = from c in candidates
                          orderby c.Vote descending //szavazatszám szerint csökkenő rendezés
                          select new                //a szükséges tulajdonságok átadása, lehetne c is
                          {
                              Name = c.Name(),
                              Part = c.Part,
                              Vote = c.Vote
                          };

            var tops = from o in ordered
                       where o.Vote == ordered.First().Vote //az első (legnagyobb) szavazattal azonosak szűrése (nem TOP 1)
                       select o;

            //7. Csoportonkénti maximumok rendezése
            Console.WriteLine("\n{0}. feladat", fSzám++);

            //rendezés első elemének kiválasztásával
            var duplarend = from o in candidates
                            orderby o.AreaCode, o.Vote descending   //rendezés két kulccsal
                            select o;

            var csoportelsők = from c in duplarend
                               group c by c.AreaCode into kerület   //csoportosítás
                               select kerület.First();              //csoportonkénti első kiválasztása

            //maximumok kikeresésével
            var maxvote = from d in candidates
                          group d by d.AreaCode into district   //csoportosítás kerületenként
                          select new
                          {
                              District = district.Key,          //kerület száma
                              Vote = (from v in district
                                      select v.Vote).Max()      //kerületenként a maximum
                          };
            var winner = from w in candidates
                         where w.Vote == (from m in maxvote
                                          where m.District == w.AreaCode
                                          select m.Vote).Single()  //a kerületen belül az egyetlen maxvote értékkel megegyező szavazatszám
                         orderby w.AreaCode                     //kerületenként rendezve
                         select new                             //visszaadott eredmény képzése, lehetne akár egy teljes kimeneti sor, vagy a w.
                         {
                             District = w.AreaCode,
                             Name = w.Name(),
                             Part = w.Part
                         };

            StreamWriter sw = new StreamWriter("kepviselok.txt");
            foreach (var w in winner)
            {
                sw.WriteLine("{0}: {1} {2}", w.District, w.Name, w.Part);
            }
            sw.Close();

            Console.WriteLine("kész");

            Console.ReadKey();
        }

    }
}
