﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace valasztas
{

    class Candidate
    {
        public int AreaCode { get; set; }
        public int Vote { get; set; }
        public string Fname { get; set; }
        public string Sname { get; set; }
        public string Part { get; set; }
        public string Name()
        {
            return Fname + " " + Sname;
        }

        public Candidate(string[] adat)
        {
            AreaCode = int.Parse(adat[0]);
            Vote = int.Parse(adat[1]);
            Fname = adat[2];
            Sname = adat[3];
            Part = adat[4] == "-" ? "Független" : adat[4];
        }
    }

    class Program
    {
        static void Main(string[] args)
        {
            //0. értékadás, pártok listája
            Dictionary<string, string> part = new Dictionary<string, string>();
            part.Add("GYEP", "Gyümölcsevők Pártja");
            part.Add("HEP", "Húsevők Pártja");
            part.Add("TISZ", "Tejivók Szövetsége");
            part.Add("ZEP", "Zöldségevők Pártja");
            part.Add("Független", "Független jelöltek");

            int fSzám = 1;

            //1. Adatok beolvasása fájlból 
            Console.WriteLine("\n{0}. feladat", fSzám++);

            List<Candidate> candidates = (
                from line in File.ReadAllLines("szavazatok.txt")
                select new Candidate(line.Split(' '))
                ).ToList();

            //2. Megszámlálás, elemszám
            Console.WriteLine("\n{0}. feladat", fSzám++);

            Console.WriteLine("A helyhatósági választáson {0} képviselőjelölt indult.", (from c in candidates select c).Count());

            //3. Keresés
            Console.WriteLine("\n{0}. feladat", fSzám++);

            Console.Write("Adja meg - szóközzel elválasztva - a keresett képviselő vezeték- és keresztnevét: ");
            string name = Console.ReadLine();

            var keres = from n in candidates
                        where (n.Name()) == name
                        select n.Vote;
            try
            {
                Console.WriteLine("A képviselő {0} szavazatot kapott.", keres.Single());
            }
            catch (Exception)
            {
                Console.WriteLine("Ilyen nevű képviselőjelölt nem szerepel a nyilvántartásban!");
            }


            //4. Összegzés
            Console.WriteLine("\n{0}. feladat", fSzám++);

            int leadott = (from s in candidates
                           select s.Vote).Sum();

            Console.WriteLine("A választáson {0} állampolgár, a jogosultak {1:P2}-a vett részt.", leadott, 1.0 * leadott / 12345);

            //5. Részösszegek - csoportonkénti aggregálás
            Console.WriteLine("\n{0}. feladat", fSzám++);

            var rate = from c in candidates
                       group c by c.Part into partok
                       select new
                       {
                           Part = partok.Key,
                           Vote = (from x in candidates
                                   where x.Part == partok.Key
                                   select x.Vote).Sum()
                       };

            //lambdával egyszerűbb
            //var rate = from c in candidates
            //           group c by c.Part into partok
            //           select new { Part = partok.Key, Vote = partok.Sum(x => x.Vote) };


            foreach (var r in rate)
            {
                Console.WriteLine("{0}: {1:P2}", part[r.Part], r.Vote * 1.0 / leadott);
            }

            //6. Maximumhelyek kigyűjtése
            Console.WriteLine("\n{0}. feladat", fSzám++);

            //rendezéssel, az elsők kigyűjtésével           
            var legek = from c in candidates
                        where c.Vote == (from v in candidates
                                         orderby v.Vote descending
                                         select v.Vote).First()
                        select new
                        {
                            Name = c.Name(),
                            Part = c.Part
                        };

            //maximummal egyezők kigyűjtése
            var maxok = from c in candidates
                        where c.Vote == (from v in candidates
                                         select v.Vote).Max()
                        select new
                        {
                            Name = c.Name(),
                            Part = c.Part
                        };

            foreach (var m in legek)
            {
                Console.WriteLine("\t{0} - {1}", m.Name, m.Part);
            }




            //7. Csoportonkénti maximumok rendezése
            Console.WriteLine("\n{0}. feladat", fSzám++);

            //rendezés első elemének kiválasztásával
            var csoportelsők = from c in
                                   (from o in candidates
                                    orderby o.AreaCode, o.Vote descending
                                    select o)
                               group c by c.AreaCode into kerület
                               select kerület.First();      //így kerületenként az első, ha az egész zárójelben lenne, akkor az első csoport (AreaCode = 1) lenne az eredmény

            //maximumok kikeresésével
            var winner = from c in candidates
                         where c.Vote == (from s in
                                              (from a in candidates
                                               group a by a.AreaCode into district
                                               select new
                                               {
                                                   District = district.Key,
                                                   Vote = (from v in district
                                                           select v.Vote).Max()
                                               })
                                          where s.District == c.AreaCode
                                          select s.Vote).Single()
                         orderby c.AreaCode
                         select new
                         {
                             District = c.AreaCode,
                             Name = c.Name(),
                             Part = c.Part
                         };


            StreamWriter sw = new StreamWriter("kepviselok.txt");
            foreach (var w in winner)
            {
                sw.WriteLine("{0}: {1} {2}", w.District, w.Name, w.Part);
            }
            sw.Close();
            Console.WriteLine("kész");

            Console.ReadKey();
        }

    }
}
