﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace valasztas
{
    public partial class Candidate
    {
        //az osztály-tábla modell kiegészítése számított értékkel
        //ezek a tagok nem használhatók a modellen alapuló lekérdezésekben, csak az eredmény rekordhalmaz-->objektum további felhasználásában
        public string Winner()   //függvény
        {
            //Párt megadása a Candidate-ben tárolt illetve a csatolt tábla használatával
            return Fname + " " + Sname + " " + PartShort;
            //return Fname + " " + Sname + " " + Part.Short;
        }
        public string Name      //tulajdonság
        {
            get
            {
                return Fname + " " + Sname;
            }
        }
    }

    class Program
    {
        static void Main(string[] args)
        {
            int fSzám = 1;

            //0. értékadás, pártok listája
            //1. Adatok beolvasása fájlból 
            Console.WriteLine("\n{0}. feladat", fSzám++);

            using (var ctx = new valasztasContainer())
            {
                #region adatbázis létrehozása
                //akkor kell létrehozni, ha a korábbitól eltérő helyen futtatjuk. pl. vizsga --> értékelés
                ctx.Database.CreateIfNotExists();
                #endregion

                #region előző futtatás rekordjainak törlése
                //ez azért kell, hogy minden indításkor üres adatbázissal induljon
                foreach (var p in ctx.PartSet)
                {
                    ctx.PartSet.Remove(p);
                }
                foreach (var c in ctx.CandidateSet)
                {
                    ctx.CandidateSet.Remove(c);
                }
                ctx.SaveChanges();
                #endregion

                Part regPart = new Part() { Short = "GYEP", Long = "Gyümölcsevők Pártja" };
                ctx.PartSet.Add(regPart);
                regPart = new Part() { Short = "HEP", Long = "Húsevők Pártja" };
                ctx.PartSet.Add(regPart);
                ctx.PartSet.Add(new Part() { Short = "TISZ", Long = "Tejivók Szövetsége" });
                ctx.PartSet.Add(new Part() { Short = "ZEP", Long = "Zöldségevők Pártja" });
                ctx.PartSet.Add(new Part() { Short = "Független", Long = "Független jelöltek" });
                ctx.SaveChanges();

                using (System.IO.StreamReader sr = new System.IO.StreamReader("szavazatok.txt"))
                {
                    while (!sr.EndOfStream)
                    {
                        string[] row = sr.ReadLine().Split(' ');
                        Candidate uj = new Candidate()
                        {
                            AreaCode = int.Parse(row[0]),
                            Vote = int.Parse(row[1]),
                            Fname = row[2],
                            Sname = row[3],
                            Part = ctx.PartSet.Find(row[4] == "-" ? "Független" : row[4])
                        };
                        ctx.CandidateSet.Add(uj);
                        ctx.SaveChanges();
                    }
                }

            }

            //2. Megszámlálás, elemszám
            Console.WriteLine("\n{0}. feladat", fSzám++);

            using (var ctx = new valasztasContainer())
            {
                Console.WriteLine(ctx.CandidateSet.Count());
            }

            //3. Keresés
            Console.WriteLine("\n{0}. feladat", fSzám++);

            Console.Write("Adja meg - szóközzel elválasztva - a keresett képviselő vezeték- és keresztnevét: ");
            string name = Console.ReadLine();

            using (var ctx = new valasztasContainer())
            {
                var find = from c in ctx.CandidateSet
                           where c.Fname + " " + c.Sname == name
                           select c;
                try
                {
                    Console.WriteLine("A {0} {1} szavazatot kapott.", find.Single().Name, find.Single().Vote);
                }
                catch (Exception)
                {
                    Console.WriteLine("Ilyen nevű képviselőjelölt nem szerepel a nyilvántartásban!");
                }
            }

            //4. Összegzés
            Console.WriteLine("\n{0}. feladat", fSzám++);

            int leadott;
            using (var ctx = new valasztasContainer())
            {

                leadott = (from s in ctx.CandidateSet
                           select s.Vote).Sum();
                Console.WriteLine("A választáson {0} állampolgár, a jogosultak {1:P2}-a vett részt.", leadott, 1.0 * leadott / 12345);
            }

            //5. Részösszegek - csoportonkénti aggregálás
            Console.WriteLine("\n{0}. feladat", fSzám++);

            using (var ctx = new valasztasContainer())
            {
                var rate = from p in ctx.PartSet
                           select new { Part = p.Long, Vote = p.Candidate.Sum(c => c.Vote) };

                foreach (var r in rate)
                {
                    Console.WriteLine("{0}: {1:P2}", r.Part, r.Vote * 1.0 / leadott);
                }
            }

            //6. Maximumhelyek kigyűjtése
            Console.WriteLine("\n{0}. feladat", fSzám++);

            using (var ctx = new valasztasContainer())
            {
                var maxs = from c in ctx.CandidateSet
                           where c.Vote == ctx.CandidateSet.Max(m => m.Vote)
                           select c;

                foreach (var m in maxs)
                {
                    Console.WriteLine(m.Winner());
                }
            }

            //7. Csoportonkénti maximumok rendezése
            Console.WriteLine("\n{0}. feladat", fSzám++);
            using (var ctx = new valasztasContainer())
            {
                var winner = from c in ctx.CandidateSet
                            orderby c.AreaCode
                            group c by c.AreaCode into District
                            let top = District.Max(m => m.Vote)
                            select new { District = District.Key, Candidate = District.FirstOrDefault(w => w.Vote == top) };

                using (System.IO.StreamWriter sw = new System.IO.StreamWriter("kepviselok.txt"))
                {
                    foreach (var w in winner)
                    {
                        sw.WriteLine("{0}: {1}", w.District, w.Candidate.Winner());
                    }
                    sw.Close();

                }

                Console.WriteLine("kész");
            }

            Console.ReadLine();

        }
    }
}
